Package: PWMEnrich
Type: Package
Title: PWM enrichment analysis
Version: 4.48.0
Date: 2015-09-25
Author: Robert Stojnic, Diego Diez
Maintainer: Diego Diez <diego10ruiz@gmail.com>
LazyLoad: yes
Description: A toolkit of high-level functions for DNA motif scanning
        and enrichment analysis built upon Biostrings. The main
        functionality is PWM enrichment analysis of already known PWMs
        (e.g. from databases such as MotifDb), but the package also
        implements high-level functions for PWM scanning and
        visualisation. The package does not perform "de novo" motif
        discovery, but is instead focused on using motifs that are
        either experimentally derived or computationally constructed by
        other tools.
License: LGPL (>= 2)
Depends: R (>= 3.5.0), methods, BiocGenerics, Biostrings
Imports: grid, seqLogo, gdata, evd, S4Vectors
Suggests: MotifDb, BSgenome, BSgenome.Dmelanogaster.UCSC.dm3,
        PWMEnrich.Dmelanogaster.background, testthat, gtools, parallel,
        PWMEnrich.Hsapiens.background, PWMEnrich.Mmusculus.background,
        BiocStyle, knitr
biocViews: MotifAnnotation, SequenceMatching, Software
VignetteBuilder: knitr
RoxygenNote: 7.1.1
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/PWMEnrich
git_branch: RELEASE_3_23
git_last_commit: a817fa3
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:01:34 UTC; biocbuild
