#' @keywords internal
"_PACKAGE"

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
#' @import BiocGenerics Biostrings methods
#' @importFrom grid grid.newpage grid.text grid.polygon grid.xaxis grid.yaxis grid.layout gpar viewport plotViewport dataViewport pushViewport popViewport unit
#' @importFrom grDevices col2rgb palette rgb
#' @importFrom graphics lines par polygon rect text
#' @importFrom stats cor dlnorm ecdf lm median na.omit optimize pchisq plnorm predict.lm qlnorm quantile
#' @importFrom utils data
#' @importFrom S4Vectors metadata
#' @importFrom seqLogo makePWM plot
#' @importFrom evd fgev pgev
#' @importFrom gdata trim
## usethis namespace: end
NULL
