### define generics, mostly for MotifEnrichmentResults

setGeneric("motifRankingForGroup", function(obj, ...) standardGeneric("motifRankingForGroup"))
setGeneric("motifRankingForSequence", function(obj, ...) standardGeneric("motifRankingForSequence"))

setGeneric("plotTopMotifsGroup", function(obj, ...) standardGeneric("plotTopMotifsGroup"))
setGeneric("plotTopMotifsSequence", function(obj, ...) standardGeneric("plotTopMotifsSequence"))

setGeneric("groupReport", function(obj, ...) standardGeneric("groupReport"))
setGeneric("sequenceReport", function(obj, ...) standardGeneric("sequenceReport"))
