% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clover.R
\name{cloverScore}
\alias{cloverScore}
\title{Calculate the Clover score using the recursive formula from Frith et al}
\usage{
cloverScore(scores, lr3 = FALSE, verbose = FALSE)
}
\arguments{
\item{scores}{a matrix of average odds scores, where columns are motifs, and rows sequences}

\item{lr3}{if to return a matrix of LR3 scores, where columns correpond to motifs, and rows to subset sizes}

\item{verbose}{if to produce verbose output of progress}
}
\value{
the LR4 score, which is the mean of LR3 scores over subset sizes
}
\description{
Calculate the Clover score using the recursive formula from Frith et al
}
