% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{colMedians}
\alias{colMedians}
\title{Calculate medians of columns}
\usage{
colMedians(x)
}
\arguments{
\item{x}{a matrix}
}
\description{
Calculate medians of columns
}
