% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pwm.R
\name{keepFinite}
\alias{keepFinite}
\title{Replace all infinite values by 0}
\usage{
keepFinite(x)
}
\arguments{
\item{x}{a vector of values}
}
\description{
Replace all infinite values by 0
}
