% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PWM-methods.R
\name{show,PWM-method}
\alias{show,PWM-method}
\title{show method for PWM}
\usage{
\S4method{show}{PWM}(object)
}
\arguments{
\item{object}{the PWM object}
}
\description{
show method for PWM
}
